--------------------------------------------------------------------------------
--	DIGITALER SINUS-GENERATOR
--
-- Modulname:			tastatur.vhd
-- Projektname:		funktionsgenerator.ise
-- Ersteller:			Thomas Brunnengrber
--	Firma:				Privat
-- Erstellt:			19.03.2006
-- Zielsystem:			Spartan 3
-- Tool versions:		XILINX ISE 8.1
-- Revision:			1.0
-- Abhngigkeiten:	keine
-- 
--------------------------------------------------------------------------------
-- Tasten werden mit Takt (ca. 5-50 ms) abgefragt.
-- Fr jede Taste ein Merker in "Tasten_gedrueckt" und in
-- "Tastenreaktion_starten".
-- Wenn Takt steigend:
--		Wenn Taste neu gedrckt (Merker in "Tasten_gedrueckt" nicht gesetzt)
-- 	=> Merker in "Tasten_gedrueckt" setzen.
--		=> Merker in "Tastenreaktion_starten" setzen.
-- 	Wenn Taste losgelassen (Merker in "Tasten_gedrueckt" gesetzt) 
-- 	=> Merker in "Tasten_gedrueckt" lschen.
--	Wenn Aktion gestartet
--	=> Merker in "Tastenreaktion_starten" sofort (asynchron) lschen.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity tastatur is
	generic ( n_Tasten : integer := 5 );	-- Anzahl der Tasten
	port ( Takt : in std_logic;				-- ca. 20-200 Hz
			 Tasten : in std_logic_vector( 1 to n_Tasten );
          Tastenreaktion_erfolgt : in std_logic;
          Tastenreaktion_starten : out std_logic_vector( 1 to n_Tasten ) );
end tastatur;

architecture a1 of tastatur is
	signal Tasten_gedrueckt : std_logic_vector( 1 to n_Tasten ) := (others => '0');
		-- Ist '1' solange Taste logisch gedrckt ist.
begin

	taste_gedrueckt_strg : process ( Takt )
	begin
		if rising_edge( Takt ) then
			for i in Tasten'RANGE(1) loop
				if Tasten( i ) = '1' then	-- Taster geschlossen:
					if Tasten_gedrueckt( i ) = '0' then
						Tasten_gedrueckt( i ) <= '1';
					end if;
				else						-- Taster nicht geschlossen:
					if Tasten_gedrueckt( i ) = '1' then
						Tasten_gedrueckt( i ) <= '0';
					end if;				
				end if;
			end loop;
		end if;
	end process;
	
	tastenreaktion_strg : process ( Takt, Tastenreaktion_erfolgt )
	begin
		if Tastenreaktion_erfolgt = '1' then
			Tastenreaktion_starten <= (others => '0');
		elsif rising_edge( Takt ) then
			for i in Tasten'RANGE(1) loop
				if Tasten( i ) = '1' then	-- Taster geschlossen:
					if Tasten_gedrueckt( i ) = '0' then
						Tastenreaktion_starten( i ) <= '1';
					end if;
				end if;
			end loop;
		end if;
	end process;

end a1;

