--------------------------------------------------------------------------------
--	DIGITALER SINUS-GENERATOR
--
-- Modulname:			user_interface.vhd
-- Projektname:		funktionsgenerator.ise
-- Ersteller:			Thomas Brunnengrber
--	Firma:				Privat
-- Erstellt:			27.03.2006
-- Zielsystem:			Spartan 3
-- Tool versions:		XILINX ISE 8.1
-- Revision:			1.0
-- Abhngigkeiten:	taktteiler2c.vhd
--							dez_to_bin_ram.vhd
--							lcd_interface.vhd
--							tastatur.vhd
-- 
--------------------------------------------------------------------------------
--	Benutzer-Schnittstelle
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;

library UNISIM;
use UNISIM.VComponents.ALL;

entity user_interface is
generic (
	n_Tasten : integer := 5;
	-- Breiten der Ausgnge:
	w_d_phase : integer := 30;		-- Breite der Phasendifferenz zwischen zwei Takten
	w_ampl : integer := 11;			-- Breite der Amplitude, 1 LSB = 1mV
--	w_offs : integer := w_ampl+1;	-- Breite der Offsetspannung, 1 LSB = 1mV, MSB = Vorzeichen
	w_max : integer := 30			-- Maximum aus allen Breiten
	-- Prfung der Eingabe:
--	u_max : integer := 4095			-- Maximal physikalisch zur Verfgung stehende
											-- Spannung in mV. Es wird ein symetrischen Ausgang
											--	angenommen, d.h. die Summe von Amplitude und
											-- Betrag des Offsets darf nicht grer sein als
											-- dieser Wert.
);
port (
	takt : in std_logic;
	reset : in std_logic;
	tasten : in std_logic_vector( 1 to n_Tasten );	-- Tasten (nicht entprellt)
	-- Signale fr Display:
	Osz_Vo		: out	std_logic;							-- Oszillator fr Spannungswandler
	LCD_Daten	: out std_logic_vector(7 downto 0); -- Datenbyte zum LCD
	LCD_RS		: out std_logic := '0';					-- LCD register select
	LCD_R_W		: out std_logic := '0';					-- LCD Read / nWrite
	LCD_Enable1	: out std_logic := '0';					-- LCD Enable
	LCD_Enable2	: out std_logic := '0';					-- LCD Enable
--test_LCD_starten : out std_logic;
--test_LCD_busy : out std_logic;
--test_Zustand : out std_logic_vector ( 12 downto 0 );
	-- Signale fr Funktionsgenerator:
--	neustart : out std_logic; -- Nur fr Generator mit einer Frequenz und fester
		-- Phasenverschiebung nach nderung der Phase oder Frequenz bentigt.
	d_phase1 : out std_logic_vector ( w_d_phase-1 downto 0 ) := conv_std_logic_vector(10000,w_d_phase);
	d_phase2 : out std_logic_vector ( w_d_phase-1 downto 0 ) := conv_std_logic_vector(100000,w_d_phase);
	ampl1, ampl2: out std_logic_vector ( w_ampl-1 downto 0 ); 
	offs1, offs2 : out std_logic_vector ( w_ampl downto 0 )
);
end entity user_interface;

architecture arch1 of user_interface is

	component taktteiler2c is
		generic ( n : integer range 1 to 30 := 2 );	-- Takt-Teilung durch 2^n 
		port ( loeschen : in std_logic;
				 Takt_ein : in std_logic;
				 Takt_aus : out std_logic );
	end component taktteiler2c;

	component tastatur is
		generic ( n_Tasten : integer := 5 );	-- Anzahl der Tasten
		port ( Takt : in std_logic;				-- ca. 20-200 Hz
				 Tasten : in std_logic_vector( 1 to n_Tasten );
				 Tastenreaktion_erfolgt : in std_logic;
				 Tastenreaktion_starten : out std_logic_vector( 1 to n_Tasten ) );
	end component tastatur;
	
	component dez_to_bin_ram is
	generic ( w_adr : integer := 6;
				 l_bin : integer := 32 );
	port ( Takt: in std_logic;
			 Start : in std_logic;
			 StartAdr : in std_logic_vector( w_adr-1 downto 0 );
			 BCD : in std_logic_vector( 3 downto 0 );
			 VZ : in std_logic;
			 NaechsteAdr : in std_logic_vector( w_adr-1 downto 0 );
			 LSD_Flag : in std_logic;
			 Adresse : out std_logic_vector( w_adr-1 downto 0 );
			 fertig : out std_logic;
			 negativ : out std_logic;
			 bin_zahl : out std_logic_vector( l_bin-1 downto 0 )
			);
	end component dez_to_bin_ram;
	
	component LCD_Interface is
	port( Takt			: in std_logic;	-- 40-50 MHz
			reset			: in std_logic;
			befehl		: in std_logic_vector( 2 downto 0 );
			position		: in std_logic_vector( 7 downto 0 ); 
			zeichen		: in std_logic_vector( 7 downto 0 ); -- Nur fr befehl = 1	
			starten		: in std_logic;			-- Befehlsausfhrung starten
			busy			: out std_logic;			-- '1', wenn LCD_Interface beschftigt
			Osz_Vo		: out	std_logic;			-- Oszillator fr Spannungswandler
			LCD_Daten	: out std_logic_vector( 7 downto 0 ); -- Datenbyte zum LCD
			LCD_RS		: out std_logic := '0';	-- LCD register select
			LCD_R_W		: out std_logic := '0';	-- LCD Read / nWrite
			LCD_Enable1	: out std_logic := '0';	-- LCD Enable Zeile 1/2
			LCD_Enable2	: out std_logic := '0'	-- LCD Enable Zeile 3/4
		 );
	end component LCD_Interface;

	-- Display-Speicher im BRAM
	--	------------------------
	-- Speicheraufbau: 64 Zeichen x 4 Zeilen x 1 Wort  32 bit
	-- 32-bit-Wort:
	-- Wenn konstantes Zeichen:
	--		8 Bit: Zeichencode
	--		-----
	--		8 Bit
	-- Wenn Stelle einer Variablen:
	--		4 Bit: Entweder 0011b (Oberes Nibble der ASCII-Zeichen "0" bis "9" ist 3)
	--				 oder 0010b (Oberes Nibble von "+" und "-" ist 2)
	--				 ASCII("+") : 2Bh=0010_1011b, ASCII("-") : 2Dh=0010_1101b
	--				 Die letzte Stelle des oberen Nibbles gibt also an, ob es sich um
	--				 ein Vorzeichen oder eine Ziffer handelt.
	--		4 Bit: Wert der Stelle (0..9) bzw. Vorzeichen.
	--		8 Bit: Neue Speicheradresse bei "Cursor links", blinkend (Variable
	--				 ausgewhlt)
	--				 oberste 2 Bit knnten aus StartAdr[i] genommen werden 
	--				 (Zeile ndert sich nicht innerhalb einer Variablen)
	--		8 Bit: Neue Speicheradresse bei "Cursor rechts", blinkend (Variable
	--				 ausgewhlt)
	--		1 Bit: LSD-Flag ('1' => Digit ist das niedrigstwertige einer Variablen)
	--		------
	--		25 Bit => 7/11 Bit Reserve
	type ram32_type is array ( 0 to 255 ) of std_logic_vector( 31 downto 0 );

	-- 0123456789ABCDEF0123456789A
	--	---------------------------
	--	Frequenz 1: 00100.00 Hz			00h
	--	A 1: 0.000 V  O 1: +2.000 V	40h
	--	Frequenz 2: 01000.00 Hz			80h
	--	A 2: 0.000 V  O 2: +2.000 V	C0h
	signal ui_ram : ram32_type := (
		x"46_000000", x"72_000000",			-- 'F', 'r'
		x"65_000000", x"71_000000",			-- 'e', 'q'
		x"75_000000", x"65_000000",			-- 'u', 'e'
		x"6E_000000", x"7A_000000",			-- 'n', 'z'
		x"20_000000", x"31_000000",			-- ' ', '1'
		x"3A_000000", x"20_000000",			-- ':', ' '
		x"30_0C_0D_00", x"30_0C_0E_00",		-- '0', '0'
		x"31_0D_0F_00", x"30_0E_10_00",		-- '1', '0'
		x"30_0F_12_00", x"2E_000000",			-- '0', '.'
		x"30_10_13_00", x"30_12_13_80",		-- '0', '0'
		x"20_000000", x"48_000000",			-- ' ', 'H'
		x"7A_000000", x"20_000000",			-- 'z', ' '
		x"20_000000", x"20_000000",			-- ' ', ' '
		x"20_000000",								-- ' '
		x"00000000", x"00000000", x"00000000", x"00000000", x"00000000",
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",	
		x"41_000000", x"20_000000",			-- 'A', ' '
		x"31_000000", x"3A_000000",			-- '1', ':'
		x"20_000000", x"30_45_47_00",			-- ' ', '0'
		x"2E_000000", x"30_45_48_00",			-- '.', '0'
		x"30_47_49_00", x"30_48_49_80",		-- '0', '0'
		x"20_000000", x"56_000000",			-- ' ', 'V'
		x"20_000000", x"20_000000",			-- ' ', ' '
		x"4F_000000", x"20_000000",			-- 'O', ' '
		x"31_000000", x"3A_000000",			-- '1', ':'
		x"20_000000", x"2B_53_54_00",			-- ' ', '+'
		x"30_53_56_00", x"2E_000000", 		-- '0', '.'
		x"30_54_57_00", x"30_56_58_00",		-- '0', '0'
		x"30_57_58_80", x"20_000000",			-- '0', ' '
		x"56_000000",								-- ' ', 'V'
		x"00000000", x"00000000", x"00000000", x"00000000", x"00000000",
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"46_000000", x"72_000000",			-- 'F', 'r'
		x"65_000000", x"71_000000",			-- 'e', 'q'
		x"75_000000", x"65_000000",			-- 'u', 'e'
		x"6E_000000", x"7A_000000",			-- 'n', 'z'
		x"20_000000", x"32_000000",			-- ' ', '2'
		x"3A_000000", x"20_000000",			-- ':', ' '
		x"30_8C_8D_00", x"31_8C_8E_00",		-- '0', '1'
		x"30_8D_8F_00", x"30_8E_90_00",		-- '0', '0'
		x"30_8F_92_00", x"2E_000000",			-- '0', '.'
		x"30_90_93_00", x"30_92_93_80",		-- '0', '0'
		x"20_000000", x"48_000000",			-- ' ', 'H'
		x"7A_000000", x"20_000000",			-- 'z', ' '
		x"20_000000", x"20_000000",			-- ' ', ' '
		x"20_000000",								-- ' '
		x"00000000", x"00000000", x"00000000", x"00000000", x"00000000",
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",	
		x"41_000000", x"20_000000",			-- 'A', ' '
		x"32_000000", x"3A_000000",			-- '2', ':'
		x"20_000000", x"30_C5_C7_00",			-- ' ', '0'
		x"2E_000000", x"30_C5_C8_00",			-- '.', '0'
		x"30_C7_C9_00", x"30_C8_C9_80",		-- '0', '0'
		x"20_000000", x"56_000000",			-- ' ', 'V'
		x"20_000000", x"20_000000",			-- ' ', ' '
		x"4F_000000", x"20_000000",			-- 'O', ' '
		x"32_000000", x"3A_000000",			-- '2', ':'
		x"20_000000", x"2B_D3_D4_00",			-- ' ', '+'
		x"30_D3_D6_00", x"2E_000000", 		-- '0', '.'
		x"30_D4_D7_00", x"30_D6_D8_00",		-- '0', '0'
		x"30_D7_D8_80", x"20_000000",			-- '0', ' '
		x"56_000000",								-- ' ', 'V'
		x"00000000", x"00000000", x"00000000", x"00000000", x"00000000",
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000",		
		x"00000000", x"00000000", x"00000000", x"00000000"	
	);
	--	Verteiltes ROM
	--	--------------
	--	Ist keine Variable ausgewhlt (Blinken aus), erfolgt die Positionierung des Cursors
	--	bei "Cursor links/rechts" entsprechend einer verteilen Tabelle (ROM):
	--		StartAdr[0] = 8-Bit Speicheradresse fr F1/F (rechteste Stelle)
	--		StartAdr[1] = 8-Bit Speicheradresse fr A1 (rechteste Stelle)
	--		StartAdr[2] = 8-Bit Speicheradresse fr O1 (rechteste Stelle)
	--		StartAdr[3] = 8-Bit Speicheradresse fr F2/PH (rechteste Stelle)
	--		StartAdr[4] = 8-Bit Speicheradresse fr A2 (rechteste Stelle)
	--		StartAdr[5] = 8-Bit Speicheradresse fr O2 (rechteste Stelle)
	--		StartAdr[6..7] : frei
	--		StartAdr[8] = 8-Bit Speicheradresse fr F1/F (linkeste Stelle)
	--		StartAdr[9] = 8-Bit Speicheradresse fr A1 (linkeste Stelle)
	--		StartAdr[10] = 8-Bit Speicheradresse fr O1 (linkeste Stelle)
	--		StartAdr[11] = 8-Bit Speicheradresse fr F2/PH (linkeste Stelle)
	--		StartAdr[12] = 8-Bit Speicheradresse fr A2 (linkeste Stelle)
	--		StartAdr[13] = 8-Bit Speicheradresse fr O2 (linkeste Stelle)
	--		StartAdr[14..15] : frei
	type rom8_type is array ( 0 to 15 ) of std_logic_vector( 7 downto 0 );
	constant start_adr_rom : rom8_type := (
		x"13",x"49",x"58",	-- F1, A1, O1 (LSD)
		x"93",x"C9",x"D8",	-- F2, A2, O2 (LSD)
		x"00",x"00",			-- frei
		x"0C",x"45",x"53",	-- F1, A1, O1 (MSD)
		x"8C",x"C5",x"D3",	-- F2, A2, O2 (MSD)
		x"00",x"00"				-- frei
	);

	signal a1, a2: std_logic_vector ( w_ampl-1 downto 0 ) := (others => '0'); 
	signal o1, o2 : std_logic_vector ( w_ampl downto 0 ) := (others => '0');
	signal Tastenreaktion_starten : std_logic_vector ( 1 to n_Tasten );
	signal Tastenreaktion_erfolgt : std_logic;
	signal ui_ram_data_ein : std_logic_vector ( 31 downto 0 );
	signal ui_ram_data_ausA : std_logic_vector ( 31 downto 0 );
	signal ui_ram_we : std_logic := '0';
	signal ui_ram_data_ausB : std_logic_vector ( 31 downto 0 );
	signal ui_ram_adrB : std_logic_vector ( 7 downto 0 );
	signal t_takt : std_logic;		-- Abtasttakt fr Tasten
	signal vk : integer range 0 to 5;		
			-- Variablenkenner: 0=F1, 1=A1, 2=O1, 3=F2, 4=A2, 5=O2
	signal cp : unsigned( 7 downto 0 );		-- Cursor-Position
	signal neuer_wert : unsigned( 3 downto 0 );
	signal alter_wert : std_logic_vector( 3 downto 0 );
	signal MSD : std_logic;
			-- 0: Adresse des LSD soll aus ROM gelesen werden.
			-- 1: Adresse des MSD soll aus ROM gelesen werden.
			-- Ist das oberste Bit der ROM-Adresse, die untere 3 sind vk.
	signal vz : std_logic;
	signal start_adr : std_logic_vector( 7 downto 0 );
	signal bin_wert : std_logic_vector( w_max-1 downto 0 );
	signal d2b_negativ : std_logic;
	signal starte_d2b : std_logic;
	signal d2b_fertig : std_logic;
	signal LCD_befehl : std_logic_vector( 2 downto 0 );
	signal LCD_starten : std_logic;
	signal LCD_busy : std_logic;
	type Zustaendetyp is ( Init1, Init2,
								  Cursor1, Cursor2, Cursor3, Cursor4,
								  Wert1, Wert2, Wert3, Wert4a, Wert4b,
								  Eingabe1a, Eingabe1b, Eingabe2,
								  Leerlauf );
	signal Zustand : Zustaendetyp;
	signal var_ausgewaehlt : std_logic;		-- 1: Variable ausgewhlt (blinkt)

begin

	tastatur_takt_generator : taktteiler2c
		generic map ( n => 20 )		-- 43MHz / 2^20 = ca. 43Hz
		port map ( loeschen => '0', Takt_ein => takt, Takt_aus => t_takt );

	d2b : dez_to_bin_ram
		generic map ( w_adr => 6, l_bin => w_max )
		port map ( Takt => takt, Start => starte_d2b, 
			StartAdr => start_adr(5 downto 0),
			BCD => ui_ram_data_ausB(27 downto 24),
			VZ => vz, -- BCD ist Vorzeichen
			NaechsteAdr => ui_ram_data_ausB(13 downto 8), 
			LSD_Flag => ui_ram_data_ausB(7),
			Adresse => ui_ram_adrB(5 downto 0),
			fertig => d2b_fertig,
			negativ => d2b_negativ,
			bin_zahl => bin_wert );

	lcd : LCD_Interface
		port map ( Takt => takt, reset => reset, befehl => LCD_befehl,
			position => std_logic_vector(cp),
			zeichen => ui_ram_data_ausA(31 downto 24),
			starten => LCD_starten, busy => LCD_busy,
			Osz_Vo => Osz_Vo, LCD_Daten => LCD_Daten,
			LCD_RS => LCD_RS,	LCD_R_W => LCD_R_W, 
			LCD_Enable1 => LCD_Enable1, LCD_Enable2 => LCD_Enable2 );
			
	t1 : tastatur
		generic map ( n_Tasten => n_Tasten )	-- Anzahl der Tasten
		port map ( Takt => t_takt, Tasten => Tasten,
				 Tastenreaktion_erfolgt => Tastenreaktion_erfolgt,
				 Tastenreaktion_starten => Tastenreaktion_starten );

	-- Dual Port RAM fr Steuerung und Display:
	ui_ram_port : process ( takt )
	begin
	   if ( rising_edge( takt ) ) then
         if ( ui_ram_we = '1') then
            ui_ram( conv_integer( cp ) ) <= ui_ram_data_ein;
				ui_ram_data_ausA <= ui_ram_data_ein;
			else
				ui_ram_data_ausA <= ui_ram( conv_integer( cp ) );		-- write first
	      end if;
			ui_ram_data_ausB <= ui_ram( conv_integer( ui_ram_adrB ) );
	   end if;
	end process;
	ui_ram_data_ein(31 downto 28) <= ui_ram_data_ausA(31 downto 28);
	ui_ram_data_ein(27 downto 24) <= std_logic_vector(alter_wert) when Zustand = Wert4a
						else std_logic_vector(neuer_wert);
	ui_ram_data_ein(23 downto 0) <= ui_ram_data_ausA(23 downto 0);	
	ui_ram_adrB(7 downto 6) <= start_adr(7 downto 6);
	vz <= not ui_ram_data_ausB(28);

	-- ROM fr Startadressen der Variablen im RAM:
	start_adr <= start_adr_rom( conv_integer( MSD & conv_std_logic_vector(vk,3) ) );
			 	
	steuerung : process ( takt, reset )
	-- Dokumentation separat in Dateien ui_ablaeufe und ui_zustaende
		variable op1 : unsigned(w_ampl-1 downto 0);	-- 11 bit
				-- Operand 1 ist A1, O1, A2 oder O2, max. zulssig: 2047
		variable test : unsigned(13 downto 0);	-- Operand 1 + neuer Wert
			-- kann bei Falscheingabe max. 2047+9999=12046 sein => 14 Bit
			-- darf max. 4095 sein => obere 2 Bit mssen 0 sein.
	begin
		if reset = '1' then
			var_ausgewaehlt <= '0';
			Zustand <= Init1;
			cp <= ( others => '0' );
			vk <= 0;
		elsif rising_edge( takt ) then
			case zustand is
				when Init1 =>
					if LCD_busy = '0' then
						if LCD_starten = '0' then
							Zustand <= Init2;
							LCD_starten <= '1';
						end if;
					else
						LCD_starten <= '0';					
					end if;
				when Init2 =>
					if LCD_busy = '0' then
						if LCD_starten = '0' then
							if cp /= 255 then
								cp <= cp+1;
								Zustand <= Init1;
								LCD_starten <= '1';
							else
								Zustand <= Cursor1;
							end if;
						end if;
					else
						LCD_starten <= '0';					
					end if;
				when Cursor1 =>
					if var_ausgewaehlt = '0' then
						cp <= unsigned( start_adr );
						Zustand <= Cursor2;
						LCD_starten <= '1';					
					else
						Zustand <= Cursor4;					
						LCD_starten <= '1';					
					end if;
				when Cursor2 =>
					if LCD_busy = '0' then
						if LCD_starten = '0' then
							Zustand <= Cursor3;
							LCD_starten <= '1';					
						end if;
					else
						LCD_starten <= '0';					
					end if;				
				when Cursor3 =>
					if LCD_busy = '0' then
						if LCD_starten = '0' then
							Zustand <= Cursor4;
							LCD_starten <= '1';					
						end if;
					else
						LCD_starten <= '0';					
					end if;				
				when Cursor4 =>
					if LCD_busy = '0' then
						if LCD_starten = '0' then
							Zustand <= Leerlauf;
						end if;
					else
						LCD_starten <= '0';					
					end if;				
				when Wert1 =>		-- Wert in RAM schreiben
					Zustand <= Wert2;
				when Wert2 =>		-- BCD/Binr-Wandler starten
					Zustand <= Wert3;
				when Wert3 =>		-- neuen Wert wandeln (warten) und dann prfen
					if d2b_fertig = '1' then
						case vk is	-- gewandelten Wert bernehmen
							when 1 =>	-- A1 soll gendert werden
								op1 := unsigned( o1( w_ampl-1 downto 0 ) );
							when 2 =>	-- O1 soll gendert werden 
								op1 := unsigned( a1 );
							when 4 =>	-- A1 soll gendert werden
								op1 := unsigned( o2( w_ampl-1 downto 0 ) );
							when 5 =>	-- O1 soll gendert werden 
								op1 := unsigned( a2 );
							when others => 
								op1 := ( others => '-' );
						end case;
						test := ("000" & op1) + unsigned( bin_wert( 13 downto 0 ) );
						if vk = 0 or vk = 3 or (test(13 downto 11) = 0) then
							LCD_starten <= '1';					
							Zustand <= Wert4b;
						else	-- Wert zu gro (bzw. zu klein)
							Zustand <= Wert4a;
						end if;
					end if;
				when Wert4a =>		-- alten Wert wieder in RAM schreiben
					Zustand <= Leerlauf;
				when Wert4b =>		-- LCD auf neuen Wert setzen
					if LCD_busy = '0' then
						if LCD_starten = '0' then
							Zustand <= Leerlauf;
						end if;
					else
						LCD_starten <= '0';					
					end if;
				when Eingabe1a =>	-- Warten, bis LCD fertig
					if LCD_busy = '0' then
						if LCD_starten = '0' then
							Zustand <= Leerlauf;
						end if;
					else
						LCD_starten <= '0';					
					end if;
				when Eingabe1b =>	-- BCD/Binr-Wandler starten
					Zustand <= Eingabe2;
				when Eingabe2 =>	-- Warten, bis LCD fertig, 	
					if LCD_busy = '0' then
						if LCD_starten = '0' then
							case vk is	-- gewandelten Wert bernehmen
								when 0 => d_phase1 <= bin_wert(w_d_phase-1 downto 0);
								when 1 => a1 <= bin_wert(w_ampl-1 downto 0);
								when 2 => o1(w_ampl-1 downto 0) <= bin_wert(w_ampl-1 downto 0);
											o1(w_ampl) <= d2b_negativ;
								when 3 => d_phase2 <= bin_wert(w_d_phase-1 downto 0);
								when 4 => a2 <= bin_wert(w_ampl-1 downto 0);
								when 5 => o2(w_ampl-1 downto 0) <= bin_wert(w_ampl-1 downto 0);
											o2(w_ampl) <= d2b_negativ;
							end case;
							Zustand <= Leerlauf;
						end if;
					else
						LCD_starten <= '0';					
					end if;
				when Leerlauf =>
					if Tastenreaktion_starten( 1 ) = '1' then		-- Eingabe
						var_ausgewaehlt <= not var_ausgewaehlt;
						if var_ausgewaehlt = '0' then
							Zustand <= Eingabe1a;
							LCD_starten <= '1';
						else
							Zustand <= Eingabe1b;
							LCD_starten <= '1';
						end if;
					elsif Tastenreaktion_starten( 2 ) = '1' then -- Wert hoch
						if var_ausgewaehlt = '1' then
							if ui_ram_data_ausA(28) = '0' then		-- Vorzeichen
								neuer_wert <= unsigned('1' & (not ui_ram_data_ausA(26 downto 25)) & '1');
							else	-- Ziffer
								if unsigned(ui_ram_data_ausA(27 downto 24)) = 9 then
									neuer_wert <= "0000";
								else
									neuer_wert <= unsigned(ui_ram_data_ausA(27 downto 24)) + 1;
								end if;
							end if;
							alter_Wert <= ui_ram_data_ausA(27 downto 24);
							Zustand <= Wert1;
						else
							Zustand <= Wert4b;	-- nur, damit Tastenreaktion_erfolgt gesetzt wird.
						end if;
					elsif Tastenreaktion_starten( 3 ) = '1' then -- Wert runter
						if var_ausgewaehlt = '1' then
							if ui_ram_data_ausA(28) = '0' then		-- Vorzeichen
								neuer_wert <= unsigned('1' & (not ui_ram_data_ausA(26 downto 25)) & '1');
							else	-- Ziffer
								if unsigned(ui_ram_data_ausA(27 downto 24)) = 0 then
									neuer_wert <= "1001";
								else
									neuer_wert <= unsigned(ui_ram_data_ausA(27 downto 24)) - 1;
								end if;
							end if;
							alter_Wert <= ui_ram_data_ausA(27 downto 24);
							Zustand <= Wert1;
						else
							Zustand <= Wert4b;	-- nur, damit Tastenreaktion_erfolgt gesetzt wird.
						end if;
					elsif Tastenreaktion_starten( 4 ) = '1' then -- Cursor rechts
						Zustand <= Cursor1;
						if var_ausgewaehlt = '1' then
							cp <= unsigned( ui_ram_data_ausA(15 downto 8) );
						else
							if vk = 5 then
								vk <= 0;
							else
								vk <= vk + 1;
							end if;
						end if;
					elsif Tastenreaktion_starten( 5 ) = '1' then -- Cursor links
						Zustand <= Cursor1;
						if var_ausgewaehlt = '1' then
							cp <= unsigned( ui_ram_data_ausA(23 downto 16) );
						else
							if vk = 0 then
								vk <= 5;
							else
								vk <= vk - 1;
							end if;
						end if;
					end if;
			end case;
		end if;
	end process;

	with Zustand select
		--	Befehl:
		--		000: Position setzen
		-- 	001: Zeichen schreiben (Position wird zunchst nach dem Schreiben
		--			vom Controller erhht, anschlieend aber wieder zurckgesetzt)
		--		100: Cursor aus, Blinken aus
		--		101: Cursor aus, Blinken ein
		-- 	110: Cursor ein, Blinken aus
	LCD_Befehl <=  "000" when Init1,
						"001" when Init2,
						"100" when Cursor2,
						"110" when Cursor3,
						"000" when Cursor4,
						"001" when Wert4b,
						"101" when Eingabe1a,
						"110" when Eingabe1b,
						"110" when Eingabe2,
						"---" when others;			-- egal	
	ui_ram_we <= '1' when Zustand = Wert1 or Zustand = Wert4a else '0';
	starte_d2b <= '1' when Zustand = Eingabe1b or Zustand = Wert2 else '0';
	MSD <= '1' when Zustand = Eingabe1b or Zustand = Eingabe2 or
						 Zustand = Wert2 or Zustand = Wert3 else '0';
	Tastenreaktion_erfolgt <= '0' when Zustand = Leerlauf else '1';

	ampl1 <= a1;
	ampl2 <= a2;
	offs1 <= o1;
	offs2 <= o2;
	
--test_LCD_starten <= LCD_starten;
--test_LCD_busy <= LCD_busy;
--with Zustand select
--test_Zustand <=	"0000000000001" when Init1,
--						"0000000000010" when Init2,
--						"0000000000100" when Cursor1,
--						"0000000001000" when Cursor2,
--						"0000000010000" when Cursor3,
--						"0000000100000" when Cursor4,
--						"0000001000000" when Wert1,
--						"0000010000000" when Wert3,
--						"0000100000000" when Eingabe1a,
--						"0001000000000" when Eingabe1b,
--						"0010000000000" when Eingabe2,
--						"1000000000000" when others;

end arch1;
